@@ -23,6 +23,8 @@
 #ifndef AVATARS_H
 #define AVATARS_H
 
+#include "file_transfers.h"
+
 #define MAX_AVATAR_FILE_SIZE 65536
 
 /* Sends avatar to friendnum.
@@ -48,5 +50,6 @@ void avatar_unset(Tox *m);
 
 void on_avatar_chunk_request(Tox *m, struct FileTransfer *ft, uint64_t position, size_t length);
 void on_avatar_file_control(Tox *m, struct FileTransfer *ft, Tox_File_Control control);
+void on_avatar_friend_connection_status(Tox *m, uint32_t friendnumber, Tox_Connection connection_status);
 
 #endif /* AVATARS_H */
