@@ -88,6 +88,9 @@ static const char *conference_cmd_list[] = {
     "/decline",
     "/exit",
     "/conference",
+#ifdef GAMES
+    "/game",
+#endif
     "/help",
     "/log",
 #ifdef AUDIO
