@@ -20,23 +20,24 @@
  *
  */
 
-#include <string.h>
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 #include <unistd.h>
 
-#include "toxic.h"
-#include "windows.h"
-#include "friendlist.h"
 #include "file_transfers.h"
+#include "friendlist.h"
 #include "line_info.h"
 #include "misc_tools.h"
 #include "notify.h"
+#include "toxic.h"
+#include "windows.h"
 
 extern FriendsList Friends;
 
 /* number of "#"'s in file transfer progress bar. Keep well below MAX_STR_SIZE */
 #define NUM_PROG_MARKS 50
+#define STR_BUF_SIZE 30
 
 /* creates initial progress line that will be updated during file transfer.
    Assumes progline has room for at least MAX_STR_SIZE bytes */
@@ -59,13 +60,15 @@ void print_progress_bar(ToxWindow *self, double bps, double pct_done, uint32_t l
         return;
     }
 
-    char pct_str[24];
+    char pct_str[STR_BUF_SIZE];
     snprintf(pct_str, sizeof(pct_str), "%.1f%%", pct_done);
 
-    char bps_str[24];
+    char bps_str[STR_BUF_SIZE];
     bytes_convert_str(bps_str, sizeof(bps_str), bps);
 
-    char prog_line[NUM_PROG_MARKS + 1] = {0};
+    char prog_line[NUM_PROG_MARKS + 1];
+    prog_line[0] = 0;
+
     int n = pct_done / (100 / NUM_PROG_MARKS);
     int i, j;
 
@@ -81,13 +84,21 @@ void print_progress_bar(ToxWindow *self, double bps, double pct_done, uint32_t l
         strcat(prog_line, "-");
     }
 
-    char full_line[strlen(pct_str) + NUM_PROG_MARKS + strlen(bps_str) + 7];
-    snprintf(full_line, sizeof(full_line), "%s [%s] %s/s", pct_str, prog_line, bps_str);
+    size_t line_buf_size = strlen(pct_str) + NUM_PROG_MARKS + strlen(bps_str) + 7;
+    char *full_line = malloc(line_buf_size);
+
+    if (full_line == NULL) {
+        return;
+    }
+
+    snprintf(full_line, line_buf_size, "%s [%s] %s/s", pct_str, prog_line, bps_str);
 
     line_info_set(self, line_id, full_line);
+
+    free(full_line);
 }
 
-static void refresh_progress_helper(ToxWindow *self, Tox *m, struct FileTransfer *ft)
+static void refresh_progress_helper(ToxWindow *self, struct FileTransfer *ft)
 {
     if (ft->state == FILE_TRANSFER_INACTIVE) {
         return;
@@ -107,33 +118,36 @@ static void refresh_progress_helper(ToxWindow *self, Tox *m, struct FileTransfer
 }
 
 /* refreshes active file transfer status bars. */
-void refresh_file_transfer_progress(ToxWindow *self, Tox *m, uint32_t friendnum)
+void refresh_file_transfer_progress(ToxWindow *self, uint32_t friendnumber)
 {
-    size_t i;
-
-    for (i = 0; i < MAX_FILES; ++i) {
-        refresh_progress_helper(self, m, &Friends.list[friendnum].file_receiver[i]);
-        refresh_progress_helper(self, m, &Friends.list[friendnum].file_sender[i]);
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        refresh_progress_helper(self, &Friends.list[friendnumber].file_receiver[i]);
+        refresh_progress_helper(self, &Friends.list[friendnumber].file_sender[i]);
     }
 }
 
-/* Returns a pointer to friendnum's FileTransfer struct associated with filenum.
- * Returns NULL if filenum is invalid.
- */
-struct FileTransfer *get_file_transfer_struct(uint32_t friendnum, uint32_t filenum)
+static void clear_file_transfer(struct FileTransfer *ft)
 {
-    size_t i;
+    *ft = (struct FileTransfer) {
+        0
+    };
+}
 
-    for (i = 0; i < MAX_FILES; ++i) {
-        struct FileTransfer *ft_send = &Friends.list[friendnum].file_sender[i];
+/* Returns a pointer to friendnumber's FileTransfer struct associated with filenumber.
+ * Returns NULL if filenumber is invalid.
+ */
+struct FileTransfer *get_file_transfer_struct(uint32_t friendnumber, uint32_t filenumber)
+{
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        struct FileTransfer *ft_send = &Friends.list[friendnumber].file_sender[i];
 
-        if (ft_send->state != FILE_TRANSFER_INACTIVE && ft_send->filenum == filenum) {
+        if (ft_send->state != FILE_TRANSFER_INACTIVE && ft_send->filenumber == filenumber) {
             return ft_send;
         }
 
-        struct FileTransfer *ft_recv = &Friends.list[friendnum].file_receiver[i];
+        struct FileTransfer *ft_recv = &Friends.list[friendnumber].file_receiver[i];
 
-        if (ft_recv->state != FILE_TRANSFER_INACTIVE && ft_recv->filenum == filenum) {
+        if (ft_recv->state != FILE_TRANSFER_INACTIVE && ft_recv->filenumber == filenumber) {
             return ft_recv;
         }
     }
@@ -144,19 +158,17 @@ struct FileTransfer *get_file_transfer_struct(uint32_t friendnum, uint32_t filen
 /* Returns a pointer to the FileTransfer struct associated with index with the direction specified.
  * Returns NULL on failure.
  */
-struct FileTransfer *get_file_transfer_struct_index(uint32_t friendnum, uint32_t index,
+struct FileTransfer *get_file_transfer_struct_index(uint32_t friendnumber, uint32_t index,
         FILE_TRANSFER_DIRECTION direction)
 {
     if (direction != FILE_TRANSFER_RECV && direction != FILE_TRANSFER_SEND) {
         return NULL;
     }
 
-    size_t i;
-
-    for (i = 0; i < MAX_FILES; ++i) {
+    for (size_t i = 0; i < MAX_FILES; ++i) {
         struct FileTransfer *ft = direction == FILE_TRANSFER_SEND ?
-                                      &Friends.list[friendnum].file_sender[i] :
-                                      &Friends.list[friendnum].file_receiver[i];
+                                      &Friends.list[friendnumber].file_sender[i] :
+                                      &Friends.list[friendnumber].file_receiver[i];
 
         if (ft->state != FILE_TRANSFER_INACTIVE && ft->index == index) {
             return ft;
@@ -169,23 +181,19 @@ struct FileTransfer *get_file_transfer_struct_index(uint32_t friendnum, uint32_t
 /* Returns a pointer to an unused file sender.
  * Returns NULL if all file senders are in use.
  */
-static struct FileTransfer *new_file_sender(ToxWindow *window, uint32_t friendnum, uint32_t filenum, uint8_t type)
+static struct FileTransfer *new_file_sender(ToxWindow *window, uint32_t friendnumber, uint32_t filenumber, uint8_t type)
 {
-    size_t i;
-
-    for (i = 0; i < MAX_FILES; ++i) {
-        struct FileTransfer *ft = &Friends.list[friendnum].file_sender[i];
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        struct FileTransfer *ft = &Friends.list[friendnumber].file_sender[i];
 
         if (ft->state == FILE_TRANSFER_INACTIVE) {
-            memset(ft, 0, sizeof(struct FileTransfer));
+            clear_file_transfer(ft);
             ft->window = window;
             ft->index = i;
-            ft->friendnum = friendnum;
-            ft->filenum = filenum;
+            ft->friendnumber = friendnumber;
+            ft->filenumber = filenumber;
             ft->file_type = type;
-            ft->last_keep_alive = get_unix_time();
             ft->state = FILE_TRANSFER_PENDING;
-            ft->direction = FILE_TRANSFER_SEND;
             return ft;
         }
     }
@@ -196,23 +204,20 @@ static struct FileTransfer *new_file_sender(ToxWindow *window, uint32_t friendnu
 /* Returns a pointer to an unused file receiver.
  * Returns NULL if all file receivers are in use.
  */
-static struct FileTransfer *new_file_receiver(ToxWindow *window, uint32_t friendnum, uint32_t filenum, uint8_t type)
+static struct FileTransfer *new_file_receiver(ToxWindow *window, uint32_t friendnumber, uint32_t filenumber,
+        uint8_t type)
 {
-    size_t i;
-
-    for (i = 0; i < MAX_FILES; ++i) {
-        struct FileTransfer *ft = &Friends.list[friendnum].file_receiver[i];
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        struct FileTransfer *ft = &Friends.list[friendnumber].file_receiver[i];
 
         if (ft->state == FILE_TRANSFER_INACTIVE) {
-            memset(ft, 0, sizeof(struct FileTransfer));
+            clear_file_transfer(ft);
             ft->window = window;
             ft->index = i;
-            ft->friendnum = friendnum;
-            ft->filenum = filenum;
+            ft->friendnumber = friendnumber;
+            ft->filenumber = filenumber;
             ft->file_type = type;
-            ft->last_keep_alive = get_unix_time();
             ft->state = FILE_TRANSFER_PENDING;
-            ft->direction = FILE_TRANSFER_RECV;
             return ft;
         }
     }
@@ -223,15 +228,15 @@ static struct FileTransfer *new_file_receiver(ToxWindow *window, uint32_t friend
 /* Initializes an unused file transfer and returns its pointer.
  * Returns NULL on failure.
  */
-struct FileTransfer *new_file_transfer(ToxWindow *window, uint32_t friendnum, uint32_t filenum,
+struct FileTransfer *new_file_transfer(ToxWindow *window, uint32_t friendnumber, uint32_t filenumber,
                                        FILE_TRANSFER_DIRECTION direction, uint8_t type)
 {
     if (direction == FILE_TRANSFER_RECV) {
-        return new_file_receiver(window, friendnum, filenum, type);
+        return new_file_receiver(window, friendnumber, filenumber, type);
     }
 
     if (direction == FILE_TRANSFER_SEND) {
-        return new_file_sender(window, friendnum, filenum, type);
+        return new_file_sender(window, friendnumber, filenumber, type);
     }
 
     return NULL;
@@ -259,7 +264,7 @@ void close_file_transfer(ToxWindow *self, Tox *m, struct FileTransfer *ft, int C
     }
 
     if (CTRL >= 0) {
-        tox_file_control(m, ft->friendnum, ft->filenum, (Tox_File_Control) CTRL, NULL);
+        tox_file_control(m, ft->friendnumber, ft->filenumber, (Tox_File_Control) CTRL, NULL);
     }
 
     if (message && self) {
@@ -269,28 +274,36 @@ void close_file_transfer(ToxWindow *self, Tox *m, struct FileTransfer *ft, int C
             box_notify(self, sound_type, NT_NOFOCUS | NT_WNDALERT_2, &self->active_box, self->name, "%s", message);
         }
 
-        line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "%s", message);
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s", message);
     }
 
-    memset(ft, 0, sizeof(struct FileTransfer));
+    clear_file_transfer(ft);
 }
 
-/* Kills all active file transfers for friendnum */
-void kill_all_file_transfers_friend(Tox *m, uint32_t friendnum)
+/* Kills active outgoing avatar file transfers for friendnumber */
+void kill_avatar_file_transfers_friend(Tox *m, uint32_t friendnumber)
 {
-    size_t i;
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        struct FileTransfer *ft = &Friends.list[friendnumber].file_sender[i];
 
-    for (i = 0; i < MAX_FILES; ++i) {
-        close_file_transfer(NULL, m, &Friends.list[friendnum].file_sender[i], TOX_FILE_CONTROL_CANCEL, NULL, silent);
-        close_file_transfer(NULL, m, &Friends.list[friendnum].file_receiver[i], TOX_FILE_CONTROL_CANCEL, NULL, silent);
+        if (ft->file_type == TOX_FILE_KIND_AVATAR) {
+            close_file_transfer(NULL, m, ft, TOX_FILE_CONTROL_CANCEL, NULL, silent);
+        }
+    }
+}
+
+/* Kills all active file transfers for friendnumber */
+void kill_all_file_transfers_friend(Tox *m, uint32_t friendnumber)
+{
+    for (size_t i = 0; i < MAX_FILES; ++i) {
+        close_file_transfer(NULL, m, &Friends.list[friendnumber].file_sender[i], TOX_FILE_CONTROL_CANCEL, NULL, silent);
+        close_file_transfer(NULL, m, &Friends.list[friendnumber].file_receiver[i], TOX_FILE_CONTROL_CANCEL, NULL, silent);
     }
 }
 
 void kill_all_file_transfers(Tox *m)
 {
-    size_t i;
-
-    for (i = 0; i < Friends.max_idx; ++i) {
+    for (size_t i = 0; i < Friends.max_idx; ++i) {
         kill_all_file_transfers_friend(m, Friends.list[i].num);
     }
 }
