@@ -29,6 +29,10 @@
 #include "toxic.h"
 #include "windows.h"
 
+#ifdef GAMES
+#include "game_base.h"
+#endif
+
 struct LastOnline {
     uint64_t last_on;
     struct tm tm;
@@ -42,6 +46,18 @@ struct ConferenceInvite {
     bool pending;
 };
 
+#ifdef GAMES
+
+struct GameInvite {
+    uint8_t *data;
+    size_t data_length;
+    GameType type;
+    uint32_t id;
+    bool pending;
+};
+
+#endif // GAMES
+
 typedef struct {
     char name[TOXIC_MAX_NAME_LENGTH + 1];
     uint16_t namelength;
@@ -59,6 +75,10 @@ typedef struct {
     struct LastOnline last_online;
     struct ConferenceInvite conference_invite;
 
+#ifdef GAMES
+    struct GameInvite game_invite;
+#endif
+
     struct FileTransfer file_receiver[MAX_FILES];
     struct FileTransfer file_sender[MAX_FILES];
 } ToxicFriend;
