@@ -38,6 +38,10 @@
 #include "toxic_strings.h"
 #include "windows.h"
 
+#ifdef GAMES
+#include "game_base.h"
+#endif
+
 extern char *DATA_FILE;
 extern ToxWindow *prompt;
 extern FriendsList Friends;
@@ -340,6 +344,63 @@ void cmd_decline(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)
     --FrndRequests.num_requests;
 }
 
+#ifdef GAMES
+
+void cmd_game(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+{
+    UNUSED_VAR(window);
+
+    if (argc < 1) {
+        game_list_print(self);
+        return;
+    }
+
+    GameType type = game_get_type(argv[1]);
+
+    if (type >= GT_Invalid) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Unknown game.");
+        return;
+    }
+
+    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
+        return;
+    }
+
+    uint32_t id = rand();
+    int ret = game_initialize(self, m, type, id, NULL, 0);
+
+    switch (ret) {
+        case 0: {
+            break;
+        }
+
+        case -1: {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Window is too small.");
+            return;
+        }
+
+        case -2: {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Game failed to initialize: Network error.");
+            return;
+
+        }
+
+        case -3: {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0,
+                          "Game is multiplayer only. Try the command again in the chat window of the contact you wish to play with.");
+            return;
+        }
+
+        default: {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Game failed to initialize (error %d)", ret);
+            return;
+        }
+    }
+}
+
+#endif // GAMES
+
 void cmd_conference(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
