@@ -20,39 +20,44 @@
  *
  */
 
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <time.h>
-#include <limits.h>
-#include <dirent.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
 #include <arpa/inet.h>
+#include <ctype.h>
+#include <limits.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
+#include <time.h>
 
-#include "toxic.h"
-#include "windows.h"
+#include "file_transfers.h"
 #include "misc_tools.h"
 #include "settings.h"
-#include "file_transfers.h"
+#include "toxic.h"
+#include "windows.h"
 
 extern ToxWindow *prompt;
 extern struct user_settings *user_settings;
 
+void clear_screen(void)
+{
+    printf("\033[2J\033[1;1H");
+}
+
 void hst_to_net(uint8_t *num, uint16_t numbytes)
 {
 #ifndef WORDS_BIGENDIAN
-    uint32_t i;
-    uint8_t buff[numbytes];
+    uint8_t *buff = malloc(numbytes);
 
-    for (i = 0; i < numbytes; ++i) {
+    if (buff == NULL) {
+        return;
+    }
+
+    for (uint32_t i = 0; i < numbytes; ++i) {
         buff[i] = num[numbytes - i - 1];
     }
 
     memcpy(num, buff, numbytes);
+    free(buff);
 #endif
-    return;
 }
 
 time_t get_unix_time(void)
@@ -66,6 +71,22 @@ int timed_out(time_t timestamp, time_t timeout)
     return timestamp + timeout <= get_unix_time();
 }
 
+/* Attempts to sleep the caller's thread for `usec` microseconds */
+void sleep_thread(long int usec)
+{
+    struct timespec req;
+    struct timespec rem;
+
+    req.tv_sec = 0;
+    req.tv_nsec = usec * 1000L;
+
+    if (nanosleep(&req, &rem) == -1) {
+        if (nanosleep(&rem, NULL) == -1) {
+            fprintf(stderr, "nanosleep() returned -1\n");
+        }
+    }
+}
+
 /* Get the current local time */
 struct tm *get_time(void)
 {
@@ -75,11 +96,16 @@ struct tm *get_time(void)
     return timeinfo;
 }
 
-/*Puts the current time in buf in the format of [HH:mm:ss] */
-void get_time_str(char *buf, int bufsize)
+/* Puts the current time in buf in the format of specified by the config */
+void get_time_str(char *buf, size_t bufsize)
 {
+    if (buf == NULL || bufsize == 0) {
+        return;
+    }
+
+    *buf = 0;
+
     if (user_settings->timestamps == TIMESTAMPS_OFF) {
-        buf[0] = '\0';
         return;
     }
 
@@ -137,11 +163,10 @@ int hex_string_to_bytes(char *buf, int size, const char *keystr)
         return -1;
     }
 
-    int i, res;
     const char *pos = keystr;
 
-    for (i = 0; i < size; ++i) {
-        res = sscanf(pos, "%2hhx", (unsigned char *)&buf[i]);
+    for (size_t i = 0; i < size; ++i) {
+        int res = sscanf(pos, "%2hhx", (unsigned char *)&buf[i]);
         pos += 2;
 
         if (res == EOF || res < 1) {
@@ -163,15 +188,31 @@ int bin_id_to_string(const char *bin_id, size_t bin_id_size, char *output, size_
         return -1;
     }
 
-    size_t i;
-
-    for (i = 0; i < TOX_ADDRESS_SIZE; ++i) {
+    for (size_t i = 0; i < TOX_ADDRESS_SIZE; ++i) {
         snprintf(&output[i * 2], output_size - (i * 2), "%02X", bin_id[i] & 0xff);
     }
 
     return 0;
 }
 
+/* Converts a binary representation of a Tox public key into a string.
+ *
+ * Returns 0 on success.
+ * Returns -1 on failure.
+ */
+int bin_pubkey_to_string(const uint8_t *bin_pubkey, size_t bin_pubkey_size, char *output, size_t output_size)
+{
+    if (bin_pubkey_size != TOX_PUBLIC_KEY_SIZE || output_size < (TOX_PUBLIC_KEY_SIZE * 2 + 1)) {
+        return -1;
+    }
+
+    for (size_t i = 0; i < TOX_PUBLIC_KEY_SIZE; ++i) {
+        snprintf(&output[i * 2], output_size - (i * 2), "%02X", bin_pubkey[i] & 0xff);
+    }
+
+    return 0;
+}
+
 /* Returns 1 if the string is empty, 0 otherwise */
 int string_is_empty(const char *string)
 {
@@ -230,43 +271,65 @@ int qsort_strcasecmp_hlpr(const void *str1, const void *str2)
     return strcasecmp((const char *) str1, (const char *) str2);
 }
 
-/* Returns 1 if nick is valid, 0 if not. A valid toxic nick:
-      - cannot be empty
-      - cannot start with a space
-      - must not contain a forward slash (for logfile naming purposes)
-      - must not contain contiguous spaces
-      - must not contain a newline or tab seqeunce */
-int valid_nick(const char *nick)
+/* case-insensitive string compare function for use with qsort */
+int qsort_ptr_char_array_helper(const void *str1, const void *str2)
 {
-    if (!nick[0] || nick[0] == ' ') {
-        return 0;
-    }
+    return strcasecmp(*(char **)str1, *(char **)str2);
+}
 
-    int i;
+static const char invalid_chars[] = {'/', '\n', '\t', '\v', '\r', '\0'};
 
-    for (i = 0; nick[i]; ++i) {
-        if ((nick[i] == ' ' && nick[i + 1] == ' ')
-                || nick[i] == '/'
-                || nick[i] == '\n'
-                || nick[i] == '\t'
-                || nick[i] == '\v'
-                || nick[i] == '\r')
+/*
+ * Helper function for `valid_nick()`.
+ *
+ * Returns true if `ch` is not in the `invalid_chars` array.
+ */
+static bool is_valid_char(char ch)
+{
+    char tmp;
 
-        {
-            return 0;
+    for (size_t i = 0; (tmp = invalid_chars[i]); ++i) {
+        if (tmp == ch) {
+            return false;
         }
     }
 
-    return 1;
+    return true;
+}
+
+/* Returns true if nick is valid.
+ *
+ * A valid toxic nick:
+ * - cannot be empty
+ * - cannot start with a space
+ * - must not contain a forward slash (for logfile naming purposes)
+ * - must not contain contiguous spaces
+ * - must not contain a newline or tab seqeunce
+ */
+bool valid_nick(const char *nick)
+{
+    if (!nick[0] || nick[0] == ' ') {
+        return false;
+    }
+
+    for (size_t i = 0; nick[i]; ++i) {
+        char ch = nick[i];
+
+        if ((ch == ' ' && nick[i + 1] == ' ') || !is_valid_char(ch)) {
+            return false;
+        }
+    }
+
+    return true;
 }
 
 /* Converts all newline/tab chars to spaces (use for strings that should be contained to a single line) */
 void filter_str(char *str, size_t len)
 {
-    size_t i;
+    for (size_t i = 0; i < len; ++i) {
+        char ch = str[i];
 
-    for (i = 0; i < len; ++i) {
-        if (str[i] == '\n' || str[i] == '\r' || str[i] == '\t' || str[i] == '\v' || str[i] == '\0') {
+        if (!is_valid_char(ch) || str[i] == '\0') {
             str[i] = ' ';
         }
     }
@@ -370,16 +433,16 @@ on_error:
     return len;
 }
 
-/* same as get_nick_truncate but for groupchats */
-int get_group_nick_truncate(Tox *m, char *buf, uint32_t peernum, uint32_t groupnum)
+/* same as get_nick_truncate but for conferences */
+int get_conference_nick_truncate(Tox *m, char *buf, uint32_t peernum, uint32_t conferencenum)
 {
     Tox_Err_Conference_Peer_Query err;
-    size_t len = tox_conference_peer_get_name_size(m, groupnum, peernum, &err);
+    size_t len = tox_conference_peer_get_name_size(m, conferencenum, peernum, &err);
 
     if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
         goto on_error;
     } else {
-        if (!tox_conference_peer_get_name(m, groupnum, peernum, (uint8_t *) buf, NULL)) {
+        if (!tox_conference_peer_get_name(m, conferencenum, peernum, (uint8_t *) buf, NULL)) {
             goto on_error;
         }
     }
@@ -491,16 +554,21 @@ bool file_exists(const char *path)
 File_Type file_type(const char *path)
 {
     struct stat s;
-    stat(path, &s);
+
+    if (stat(path, &s) == -1) {
+        return FILE_TYPE_OTHER;
+    }
 
     switch (s.st_mode & S_IFMT) {
         case S_IFDIR:
             return FILE_TYPE_DIRECTORY;
+
         case S_IFREG:
             return FILE_TYPE_REGULAR;
+
         default:
             return FILE_TYPE_OTHER;
-   }
+    }
 }
 
 /* returns file size. If file doesn't exist returns 0. */
@@ -515,34 +583,17 @@ off_t file_size(const char *path)
     return st.st_size;
 }
 
-/* compares the first size bytes of fp to signature.
-   Returns 0 if they are the same, 1 if they differ, and -1 on error.
-
-   On success this function will seek back to the beginning of fp */
-int check_file_signature(const char *signature, size_t size, FILE *fp)
-{
-    char buf[size];
-
-    if (fread(buf, size, 1, fp) != 1) {
-        return -1;
-    }
-
-    int ret = memcmp(signature, buf, size);
-
-    if (fseek(fp, 0L, SEEK_SET) == -1) {
-        return -1;
-    }
-
-    return ret == 0 ? 0 : 1;
-}
-
 /* sets window title in tab bar. */
 void set_window_title(ToxWindow *self, const char *title, int len)
 {
+    if (len <= 0 || !title) {
+        return;
+    }
+
     char cpy[TOXIC_MAX_NAME_LENGTH + 1];
 
-    if (self->is_groupchat) { /* keep groupnumber in title */
-        snprintf(cpy, sizeof(cpy), "%d %s", self->num, title);
+    if (self->type == WINDOW_TYPE_CONFERENCE) { /* keep conferencenumber in title for invites */
+        snprintf(cpy, sizeof(cpy), "%u %s", self->num, title);
     } else {
         snprintf(cpy, sizeof(cpy), "%s", title);
     }
@@ -555,35 +606,49 @@ void set_window_title(ToxWindow *self, const char *title, int len)
     snprintf(self->name, sizeof(self->name), "%s", cpy);
 }
 
-/* Return true if address appears to be a valid ipv4 address. */
-bool is_ip4_address(const char *address)
+/*
+ * Frees all members of a pointer array plus `arr`.
+ */
+void free_ptr_array(void **arr)
 {
-    struct sockaddr_in s_addr;
-    return inet_pton(AF_INET, address, &(s_addr.sin_addr)) != 0;
+    if (arr == NULL) {
+        return;
+    }
+
+    void **tmp = arr;
+
+    while (*arr) {
+        free(*arr);
+        ++arr;
+    }
+
+    free(tmp);
 }
 
-/* Return true if address roughly appears to be a valid ipv6 address.
+/*
+ * Returns a null terminated array of `length` pointers. Each pointer is allocated `bytes` bytes.
+ * Returns NULL on failure.
  *
- * TODO: Improve this function (inet_pton behaves strangely with ipv6).
- * for now the only guarantee is that it won't return true if the
- * address is a domain or ipv4 address, and should only be used if you're
- * reasonably sure that the address is one of the three (ipv4, ipv6 or a domain).
+ * The caller is responsible for freeing the array with `free_ptr_array`.
  */
-bool is_ip6_address(const char *address)
+void **malloc_ptr_array(size_t length, size_t bytes)
 {
-    size_t i;
-    size_t num_colons = 0;
-    char ch = 0;
+    void **arr = malloc((length + 1) * sizeof(void *));
 
-    for (i = 0; (ch = address[i]); ++i) {
-        if (ch == '.') {
-            return false;
-        }
+    if (arr == NULL) {
+        return NULL;
+    }
 
-        if (ch == ':') {
-            ++num_colons;
+    for (size_t i = 0; i < length; ++i) {
+        arr[i] = malloc(bytes);
+
+        if (arr[i] == NULL) {
+            free_ptr_array(arr);
+            return NULL;
         }
     }
 
-    return num_colons > 1 && num_colons < 8;
+    arr[length] = NULL;
+
+    return arr;
 }
