@@ -60,6 +60,9 @@ static const char *glob_cmd_list[] = {
     "/decline",
     "/exit",
     "/conference",
+#ifdef GAMES
+    "/game",
+#endif
     "/help",
     "/log",
     "/myid",
@@ -286,7 +289,10 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
             if (wcs_to_mbs_buf(line, ctx->line, MAX_STR_SIZE) == -1) {
                 line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
             } else {
-                line_info_add(self, false, NULL, NULL, PROMPT, 0, 0, "%s", line);
+                if (strcmp(line, "/clear") != 0) {
+                    line_info_add(self, false, NULL, NULL, PROMPT, 0, 0, "%s", line);
+                }
+
                 execute(ctx->history, self, m, line, GLOBAL_COMMAND_MODE);
             }
         }
@@ -321,8 +327,6 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
         mvwprintw(ctx->linewin, 0, 0, "%ls", &ctx->line[ctx->start]);
     }
 
-    mvwhline(ctx->linewin, 0, ctx->len, ' ', x2 - ctx->len);
-
     curs_set(1);
 
     StatusBar *statusbar = self->stb;
