@@ -20,23 +20,23 @@
  *
  */
 
+#include <ctype.h>
+#include <libconfig.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libconfig.h>
-#include <ctype.h>
 
+#include "configdir.h"
+#include "misc_tools.h"
+#include "notify.h"
 #include "toxic.h"
 #include "windows.h"
-#include "configdir.h"
-#include "notify.h"
-#include "misc_tools.h"
 
 #ifdef AUDIO
 #include "audio_device.h"
 #endif /* AUDIO */
 
-#include "settings.h"
 #include "line_info.h"
+#include "settings.h"
 
 #ifndef PACKAGE_DATADIR
 #define PACKAGE_DATADIR "."
@@ -58,11 +58,13 @@ static struct ui_strings {
     const char *native_colors;
     const char *autolog;
     const char *history_size;
+    const char *notification_timeout;
     const char *show_typing_self;
     const char *show_typing_other;
     const char *show_welcome_msg;
     const char *show_connection_msg;
     const char *nodeslist_update_freq;
+    const char *autosave_freq;
 
     const char *line_join;
     const char *line_quit;
@@ -71,6 +73,11 @@ static struct ui_strings {
 
     const char *mplex_away;
     const char *mplex_away_note;
+
+    const char *color_bar_bg;
+    const char *color_bar_fg;
+    const char *color_bar_accent;
+    const char *color_bar_notify;
 } ui_strings = {
     "ui",
     "timestamps",
@@ -85,17 +92,23 @@ static struct ui_strings {
     "native_colors",
     "autolog",
     "history_size",
+    "notification_timeout",
     "show_typing_self",
     "show_typing_other",
     "show_welcome_msg",
     "show_connection_msg",
     "nodeslist_update_freq",
+    "autosave_freq",
     "line_join",
     "line_quit",
     "line_alert",
     "line_normal",
     "mplex_away",
     "mplex_away_note",
+    "color_bar_bg",
+    "color_bar_fg",
+    "color_bar_accent",
+    "color_bar_notify",
 };
 
 static void ui_defaults(struct user_settings *settings)
@@ -112,11 +125,13 @@ static void ui_defaults(struct user_settings *settings)
     settings->bell_on_invite = 0;
     settings->colour_theme = DFLT_COLS;
     settings->history_size = 700;
+    settings->notification_timeout = 6000;
     settings->show_typing_self = SHOW_TYPING_ON;
     settings->show_typing_other = SHOW_TYPING_ON;
     settings->show_welcome_msg = SHOW_WELCOME_MSG_ON;
     settings->show_connection_msg = SHOW_CONNECTION_MSG_ON;
     settings->nodeslist_update_freq = 7;
+    settings->autosave_freq = 600;
 
     snprintf(settings->line_join, LINE_HINT_MAX + 1, "%s", LINE_JOIN);
     snprintf(settings->line_quit, LINE_HINT_MAX + 1, "%s", LINE_QUIT);
@@ -124,10 +139,7 @@ static void ui_defaults(struct user_settings *settings)
     snprintf(settings->line_normal, LINE_HINT_MAX + 1, "%s", LINE_NORMAL);
 
     settings->mplex_away = MPLEX_ON;
-    snprintf(settings->mplex_away_note,
-             sizeof(settings->mplex_away_note),
-             "%s",
-             MPLEX_AWAY_NOTE);
+    snprintf(settings->mplex_away_note, sizeof(settings->mplex_away_note), "%s", MPLEX_AWAY_NOTE);
 }
 
 static const struct keys_strings {
@@ -139,8 +151,6 @@ static const struct keys_strings {
     const char *half_page_up;
     const char *half_page_down;
     const char *page_bottom;
-    const char *peer_list_up;
-    const char *peer_list_down;
     const char *toggle_peerlist;
     const char *toggle_pastemode;
 } key_strings = {
@@ -152,8 +162,6 @@ static const struct keys_strings {
     "half_page_up",
     "half_page_down",
     "page_bottom",
-    "peer_list_up",
-    "peer_list_down",
     "toggle_peerlist",
     "toggle_paste_mode",
 };
@@ -168,8 +176,6 @@ static void key_defaults(struct user_settings *settings)
     settings->key_half_page_up = T_KEY_C_F;
     settings->key_half_page_down = T_KEY_C_V;
     settings->key_page_bottom = T_KEY_C_H;
-    settings->key_peer_list_up = T_KEY_C_LB;
-    settings->key_peer_list_down = T_KEY_C_RB;
     settings->key_toggle_peerlist = T_KEY_C_B;
     settings->key_toggle_pastemode = T_KEY_C_T;
 }
@@ -204,19 +210,28 @@ static const struct audio_strings {
     const char *self;
     const char *input_device;
     const char *output_device;
-    const char *VAD_treshold;
+    const char *VAD_threshold;
+    const char *conference_audio_channels;
+    const char *chat_audio_channels;
+    const char *push_to_talk;
 } audio_strings = {
     "audio",
     "input_device",
     "output_device",
-    "VAD_treshold",
+    "VAD_threshold",
+    "conference_audio_channels",
+    "chat_audio_channels",
+    "push_to_talk",
 };
 
 static void audio_defaults(struct user_settings *settings)
 {
     settings->audio_in_dev = 0;
     settings->audio_out_dev = 0;
-    settings->VAD_treshold = 40.0;
+    settings->VAD_threshold = 5.0;
+    settings->conference_audio_channels = 1;
+    settings->chat_audio_channels = 2;
+    settings->push_to_talk = 0;
 }
 #endif
 
@@ -330,7 +345,7 @@ int settings_load(struct user_settings *s, const char *patharg)
 
         if (config_setting_lookup_int(setting, ui_strings.time_format, &time)) {
             if (time == 12) {
-                snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", "%I:%M:%S %p");
+                snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", "%I:%M %p");
                 snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", "%Y/%m/%d [%I:%M:%S %p]");
             }
         }
@@ -339,6 +354,22 @@ int settings_load(struct user_settings *s, const char *patharg)
             snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", str);
         }
 
+        if (config_setting_lookup_string(setting, ui_strings.color_bar_bg, &str)) {
+            snprintf(s->color_bar_bg, sizeof(s->color_bar_bg), "%s", str);
+        }
+
+        if (config_setting_lookup_string(setting, ui_strings.color_bar_fg, &str)) {
+            snprintf(s->color_bar_fg, sizeof(s->color_bar_fg), "%s", str);
+        }
+
+        if (config_setting_lookup_string(setting, ui_strings.color_bar_accent, &str)) {
+            snprintf(s->color_bar_accent, sizeof(s->color_bar_accent), "%s", str);
+        }
+
+        if (config_setting_lookup_string(setting, ui_strings.color_bar_notify, &str)) {
+            snprintf(s->color_bar_notify, sizeof(s->color_bar_notify), "%s", str);
+        }
+
         if (config_setting_lookup_string(setting, ui_strings.log_timestamp_format, &str)) {
             snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", str);
         }
@@ -369,7 +400,9 @@ int settings_load(struct user_settings *s, const char *patharg)
         config_setting_lookup_bool(setting, ui_strings.show_connection_msg, &s->show_connection_msg);
 
         config_setting_lookup_int(setting, ui_strings.history_size, &s->history_size);
+        config_setting_lookup_int(setting, ui_strings.notification_timeout, &s->notification_timeout);
         config_setting_lookup_int(setting, ui_strings.nodeslist_update_freq, &s->nodeslist_update_freq);
+        config_setting_lookup_int(setting, ui_strings.autosave_freq, &s->autosave_freq);
 
         if (config_setting_lookup_string(setting, ui_strings.line_join, &str)) {
             snprintf(s->line_join, sizeof(s->line_join), "%s", str);
@@ -485,14 +518,6 @@ int settings_load(struct user_settings *s, const char *patharg)
             set_key_binding(&s->key_page_bottom, &tmp);
         }
 
-        if (config_setting_lookup_string(setting, key_strings.peer_list_up, &tmp)) {
-            set_key_binding(&s->key_peer_list_up, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.peer_list_down, &tmp)) {
-            set_key_binding(&s->key_peer_list_down, &tmp);
-        }
-
         if (config_setting_lookup_string(setting, key_strings.toggle_peerlist, &tmp)) {
             set_key_binding(&s->key_toggle_peerlist, &tmp);
         }
@@ -511,7 +536,16 @@ int settings_load(struct user_settings *s, const char *patharg)
         config_setting_lookup_int(setting, audio_strings.output_device, &s->audio_out_dev);
         s->audio_out_dev = s->audio_out_dev < 0 || s->audio_out_dev > MAX_DEVICES ? 0 : s->audio_out_dev;
 
-        config_setting_lookup_float(setting, audio_strings.VAD_treshold, &s->VAD_treshold);
+        config_setting_lookup_float(setting, audio_strings.VAD_threshold, &s->VAD_threshold);
+
+        config_setting_lookup_int(setting, audio_strings.conference_audio_channels, &s->conference_audio_channels);
+        s->conference_audio_channels = s->conference_audio_channels <= 0
+                                       || s->conference_audio_channels > 2 ? 1 : s->conference_audio_channels;
+
+        config_setting_lookup_int(setting, audio_strings.chat_audio_channels, &s->chat_audio_channels);
+        s->chat_audio_channels = s->chat_audio_channels <= 0 || s->chat_audio_channels > 2 ? 2 : s->chat_audio_channels;
+
+        config_setting_lookup_bool(setting, audio_strings.push_to_talk, &s->push_to_talk);
     }
 
 #endif
