@@ -256,7 +256,7 @@ static void init_term(void)
     keypad(stdscr, 1);
     noecho();
     nonl();
-    timeout(50);
+    timeout(30);
 
     if (has_colors()) {
         short bg_color = COLOR_BLACK;
@@ -376,8 +376,10 @@ static void init_term(void)
         init_pair(YELLOW, COLOR_YELLOW, bg_color);
         init_pair(MAGENTA, COLOR_MAGENTA, bg_color);
         init_pair(BLACK, COLOR_BLACK, COLOR_BLACK);
-        init_pair(BLUE_BLACK, COLOR_BLUE, COLOR_BLACK);
+        init_pair(WHITE_BLUE, COLOR_WHITE, COLOR_BLUE);
         init_pair(BLACK_WHITE, COLOR_BLACK, COLOR_WHITE);
+        init_pair(WHITE_BLACK, COLOR_WHITE, COLOR_BLACK);
+        init_pair(WHITE_GREEN, COLOR_WHITE, COLOR_GREEN);
         init_pair(BLACK_BG, COLOR_BLACK, bar_bg_color);
         init_pair(PURPLE_BG, COLOR_MAGENTA, bar_bg_color);
         init_pair(BAR_TEXT, bar_fg_color, bar_bg_color);
@@ -804,6 +806,7 @@ static void init_tox_callbacks(Tox *m)
     tox_callback_file_chunk_request(m, on_file_chunk_request);
     tox_callback_file_recv_control(m, on_file_recv_control);
     tox_callback_file_recv_chunk(m, on_file_recv_chunk);
+    tox_callback_friend_lossless_packet(m, on_lossless_custom_packet);
 }
 
 static void init_tox_options(struct Tox_Options *tox_opts)
@@ -1461,6 +1464,8 @@ int main(int argc, char **argv)
     /* Make sure all written files are read/writeable only by the current user. */
     umask(S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 
+    srand(time(NULL)); // We use rand() for trivial/non-security related things
+
     parse_args(argc, argv);
 
     /* Use the -b flag to enable stderr */
