@@ -20,40 +20,40 @@
  *
  */
 
-#include "video_device.h"
 #include "video_call.h"
+#include "video_device.h"
 
 #include <sys/ioctl.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xos.h>
 
 #include <vpx/vpx_image.h>
 
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 #import "osx_video.h"
 #else
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
 #include <fcntl.h>
+#include <sys/mman.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <X11/Xlib.h>
+#include <X11/Xos.h>
+#include <X11/Xutil.h>
 #if defined(__OpenBSD__) || defined(__NetBSD__)
 #include <sys/videoio.h>
 #else
 #include <linux/videodev2.h>
 #endif /* defined(__OpenBSD__) || defined(__NetBSD__) */
-#endif /* __OSX__ */
+#endif /* __OSX__ || __APPLE__ */
 
 #include "line_info.h"
+#include "misc_tools.h"
 #include "settings.h"
 
 #include <errno.h>
-
-#include <stdbool.h>
-#include <string.h>
 #include <pthread.h>
-#include <unistd.h>
+#include <stdbool.h>
 #include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #ifdef VIDEO
 
@@ -71,7 +71,7 @@ typedef struct VideoDevice {
     void *cb_data;                          /* Data to be passed to callback */
     int32_t friend_number;                  /* ToxAV friend number */
 
-#if !defined(__OSX__)
+#if !(defined(__OSX__) || defined(__APPLE__))
     int fd;                                 /* File descriptor of video device selected/opened */
     struct v4l2_format fmt;
     struct VideoBuffer *buffers;
@@ -101,8 +101,8 @@ uint32_t primary_video_device[2];          /* Primary device */
 static ToxAV *av = NULL;
 
 /* q_mutex */
-#define lock pthread_mutex_lock(&video_mutex);
-#define unlock pthread_mutex_unlock(&video_mutex);
+#define lock pthread_mutex_lock(&video_mutex)
+#define unlock pthread_mutex_unlock(&video_mutex)
 pthread_mutex_t video_mutex;
 
 bool video_thread_running = true,
@@ -136,7 +136,7 @@ static void yuv420tobgr(uint16_t width, uint16_t height, const uint8_t *y,
     }
 }
 
-#if !defined(__OSX__)
+#if !(defined(__OSX__) || defined(__APPLE__))
 static void yuv422to420(uint8_t *plane_y, uint8_t *plane_u, uint8_t *plane_v,
                         uint8_t *input, uint16_t width, uint16_t height)
 {
@@ -180,18 +180,18 @@ static int xioctl(int fh, unsigned long request, void *arg)
 #ifdef VIDEO
 VideoDeviceError init_video_devices(ToxAV *av_)
 #else
-VideoDeviceError init_video_devices()
+VideoDeviceError init_video_devices(void)
 #endif /* VIDEO */
 {
     size[vdt_input] = 0;
 
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 
     if (osx_video_init((char **)video_devices_names[vdt_input], &size[vdt_input]) != 0) {
         return vde_InternalError;
     }
 
-#else /* not __OSX__*/
+#else /* not __OSX__ || __APPLE__ */
 
     for (; size[vdt_input] <= MAX_DEVICES; ++size[vdt_input]) {
         int fd;
@@ -257,7 +257,7 @@ VideoDeviceError terminate_video_devices(void)
     video_thread_running = false;
     unlock;
 
-    usleep(20000);
+    sleep_thread(20000L);
 
     int i;
 
@@ -269,9 +269,9 @@ VideoDeviceError terminate_video_devices(void)
         return (VideoDeviceError) vde_InternalError;
     }
 
-#ifdef __OSX__
+#if defined(__OSX__) || defined(__APPLE__)
     osx_video_release();
-#endif /* __OSX__ */
+#endif /* __OSX__ || __APPLE__ */
 
     return (VideoDeviceError) vde_None;
 }
@@ -279,13 +279,13 @@ VideoDeviceError terminate_video_devices(void)
 VideoDeviceError register_video_device_callback(int32_t friend_number, uint32_t device_idx,
         VideoDataHandleCallback callback, void *data)
 {
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 
     if (size[vdt_input] <= device_idx || !video_devices_running[vdt_input][device_idx]) {
         return vde_InvalidSelection;
     }
 
-#else /* not __OSX__ */
+#else /* not __OSX__ || __APPLE__ */
 
     if (size[vdt_input] <= device_idx || !video_devices_running[vdt_input][device_idx]
             || !video_devices_running[vdt_input][device_idx]->fd) {
@@ -314,9 +314,10 @@ VideoDeviceError set_primary_video_device(VideoDeviceType type, int32_t selectio
     return vde_None;
 }
 
-VideoDeviceError open_primary_video_device(VideoDeviceType type, uint32_t *device_idx)
+VideoDeviceError open_primary_video_device(VideoDeviceType type, uint32_t *device_idx,
+        uint32_t *width, uint32_t *height)
 {
-    return open_video_device(type, primary_video_device[type], device_idx);
+    return open_video_device(type, primary_video_device[type], device_idx, width, height);
 }
 
 void get_primary_video_device_name(VideoDeviceType type, char *buf, int size)
@@ -324,7 +325,8 @@ void get_primary_video_device_name(VideoDeviceType type, char *buf, int size)
     memcpy(buf, dvideo_device_names[type], size);
 }
 
-VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint32_t *device_idx)
+VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint32_t *device_idx,
+                                   uint32_t *width, uint32_t *height)
 {
     if (size[type] <= selection || selection < 0) {
         return vde_InvalidSelection;
@@ -369,15 +371,16 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
     if (type == vdt_input) {
         video_thread_paused = true;
 
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 
+        /* TODO: use requested resolution */
         if (osx_video_open_device(selection, &device->video_width, &device->video_height) != 0) {
             free(device);
             unlock;
             return vde_FailedStart;
         }
 
-#else /* not __OSX__*/
+#else /* not __OSX__ || __APPLE__ */
         /* Open selected device */
         char device_address[] = "/dev/videoXX";
         snprintf(device_address + 10, sizeof(device_address) - 10, "%i", selection);
@@ -400,11 +403,12 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
         }
 
         /* Setup video format */
-        struct v4l2_format fmt;
-        memset(&(fmt), 0, sizeof(fmt));
+        struct v4l2_format fmt = {0};
 
         fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
+        fmt.fmt.pix.width = width == NULL ? 0 : *width;
+        fmt.fmt.pix.height = height == NULL ? 0 : *height;
 
         if (-1 == xioctl(device->fd, VIDIOC_S_FMT, &fmt)) {
             close(device->fd);
@@ -417,8 +421,8 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
         device->video_height = fmt.fmt.pix.height;
 
         /* Request buffers */
-        struct v4l2_requestbuffers req;
-        memset(&(req), 0, sizeof(req));
+        struct v4l2_requestbuffers req = {0};
+
         req.count = 4;
         req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         req.memory = V4L2_MEMORY_MMAP;
@@ -440,8 +444,7 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
         device->buffers = calloc(req.count, sizeof(struct VideoBuffer));
 
         for (i = 0; i < req.count; ++i) {
-            struct v4l2_buffer buf;
-            memset(&(buf), 0, sizeof(buf));
+            struct v4l2_buffer buf = {0};
 
             buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
             buf.memory = V4L2_MEMORY_MMAP;
@@ -478,8 +481,7 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
         enum v4l2_buf_type type;
 
         for (i = 0; i < device->n_buffers; ++i) {
-            struct v4l2_buffer buf;
-            memset(&(buf), 0, sizeof(buf));
+            struct v4l2_buffer buf = {0};
 
             buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
             buf.memory = V4L2_MEMORY_MMAP;
@@ -545,6 +547,14 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
 
         vpx_img_alloc(&device->input, VPX_IMG_FMT_I420, device->video_width, device->video_height, 1);
 
+        if (width != NULL) {
+            *width = device->video_width;
+        }
+
+        if (height != NULL) {
+            *height = device->video_height;
+        }
+
         video_thread_paused = false;
     } else { /* vdt_output */
 
@@ -596,6 +606,8 @@ VideoDeviceError write_video_out(uint16_t width, uint16_t height,
                                  int32_t ystride, int32_t ustride, int32_t vstride,
                                  void *user_data)
 {
+    UNUSED_VAR(user_data);
+
     VideoDevice *device = video_devices_running[vdt_output][0];
 
     if (!device) {
@@ -659,7 +671,7 @@ void *video_thread_poll(void *arg)  // TODO: maybe use thread for every input so
     /*
      * NOTE: We only need to poll input devices for data.
      */
-    (void)arg;
+    UNUSED_VAR(arg);
     uint32_t i;
 
     while (1) {
@@ -673,7 +685,7 @@ void *video_thread_poll(void *arg)  // TODO: maybe use thread for every input so
         unlock;
 
         if (video_thread_paused) {
-            usleep(10000);    /* Wait for unpause. */
+            sleep_thread(10000L);    /* Wait for unpause. */
         } else {
             for (i = 0; i < size[vdt_input]; ++i) {
                 lock;
@@ -687,16 +699,15 @@ void *video_thread_poll(void *arg)  // TODO: maybe use thread for every input so
                     uint8_t *u = device->input.planes[1];
                     uint8_t *v = device->input.planes[2];
 
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 
                     if (osx_video_read_device(y, u, v, &video_width, &video_height) != 0) {
                         unlock;
                         continue;
                     }
 
-#else /* not __OSX__*/
-                    struct v4l2_buffer buf;
-                    memset(&(buf), 0, sizeof(buf));
+#else /* not __OSX__ || __APPLE__ */
+                    struct v4l2_buffer buf = {0};
 
                     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                     buf.memory = V4L2_MEMORY_MMAP;
@@ -748,7 +759,7 @@ void *video_thread_poll(void *arg)  // TODO: maybe use thread for every input so
                     XFlush(device->x_display);
                     free(img_data);
 
-#if !defined(__OSX__)
+#if !(defined(__OSX__) || defined(__APPLE__))
 
                     if (-1 == xioctl(device->fd, VIDIOC_QBUF, &buf)) {
                         unlock;
@@ -762,7 +773,8 @@ void *video_thread_poll(void *arg)  // TODO: maybe use thread for every input so
                 unlock;
             }
 
-            usleep(1000 * 1000 / 24);
+            long int sleep_duration = 1000 * 1000 / 24;
+            sleep_thread(sleep_duration);
         }
     }
 
@@ -789,10 +801,10 @@ VideoDeviceError close_video_device(VideoDeviceType type, uint32_t device_idx)
     if (!device->ref_count) {
 
         if (type == vdt_input) {
-#if defined(__OSX__)
+#if defined(__OSX__) || defined(__APPLE__)
 
             osx_video_close_device(device_idx);
-#else /* not __OSX__ */
+#else /* not __OSX__ || __APPLE__ */
             enum v4l2_buf_type buf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
             if (-1 == xioctl(device->fd, VIDIOC_STREAMOFF, &buf_type)) {}
@@ -813,9 +825,9 @@ VideoDeviceError close_video_device(VideoDeviceType type, uint32_t device_idx)
             XCloseDisplay(device->x_display);
             pthread_mutex_destroy(device->mutex);
 
-#if !defined(__OSX__)
+#if !(defined(__OSX__) || defined(__APPLE__))
             free(device->buffers);
-#endif /* not __OSX__ */
+#endif /* not __OSX__ || __APPLE__ */
 
             free(device);
         } else {
@@ -840,7 +852,7 @@ void print_video_devices(ToxWindow *self, VideoDeviceType type)
     int i;
 
     for (i = 0; i < size[type]; ++i) {
-        line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "%d: %s", i, video_devices_names[type][i]);
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%d: %s", i, video_devices_names[type][i]);
     }
 
     return;
