@@ -37,6 +37,10 @@
 #include "toxic.h"
 #include "windows.h"
 
+#ifdef GAMES
+#include "game_base.h"
+#endif
+
 extern char *DATA_FILE;
 extern struct Winthread Winthread;
 
@@ -322,6 +326,53 @@ void on_friend_read_receipt(Tox *m, uint32_t friendnumber, uint32_t receipt, voi
         }
     }
 }
+
+void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *data, size_t length, void *userdata)
+{
+    UNUSED_VAR(userdata);
+
+    if (length == 0 || data == NULL) {
+        return;
+    }
+
+    uint8_t type = data[0];
+
+    switch (type) {
+#ifdef GAMES
+
+        case CUSTOM_PACKET_GAME_INVITE: {
+            for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
+                ToxWindow *window = windows[i];
+
+                if (window != NULL && window->onGameInvite != NULL) {
+                    window->onGameInvite(window, m, friendnumber, data + 1, length - 1);
+                }
+            }
+
+            break;
+        }
+
+        case CUSTOM_PACKET_GAME_DATA: {
+            for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
+                ToxWindow *window = windows[i];
+
+                if (window != NULL && window->onGameData != NULL) {
+                    window->onGameData(window, m, friendnumber, data + 1, length - 1);
+                }
+            }
+
+            break;
+        }
+
+#endif // GAMES
+
+        default: {
+            fprintf(stderr, "Got unknown custom packet of type: %u\n", type);
+            return;
+        }
+    }
+}
+
 /* CALLBACKS END */
 
 int add_window(Tox *m, ToxWindow *w)
@@ -367,7 +418,9 @@ void set_active_window_index(uint8_t index)
     }
 }
 
-/* Shows next window when tab or back-tab is pressed */
+/* Displays the next window if `ch` is equal to the next window key binding.
+ * Otherwise displays the previous window.
+ */
 void set_next_window(int ch)
 {
     if (ch == user_settings->key_next_tab) {
@@ -394,8 +447,6 @@ void set_next_window(int ch)
 /* Deletes window w and cleans up */
 void del_window(ToxWindow *w)
 {
-    set_active_window_index(0);
-
     uint8_t idx = w->index;
     delwin(w->window_bar);
     delwin(w->window);
@@ -404,7 +455,16 @@ void del_window(ToxWindow *w)
 
     clear();
     refresh();
-    --num_active_windows;
+
+    if (num_active_windows > 0) {
+        if (active_window_index == 2) {    // if closing current window would bring us to friend list
+            set_next_window(-1);           // skip back to the home window instead. FIXME: magic numbers
+        }
+
+        set_next_window(-1);
+
+        --num_active_windows;
+    }
 }
 
 ToxWindow *init_windows(Tox *m)
@@ -436,6 +496,9 @@ void on_window_resize(void)
     refresh();
     clear();
 
+    int x2;
+    int y2;
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         ToxWindow *w = windows[i];
 
@@ -451,6 +514,29 @@ void on_window_resize(void)
             continue;
         }
 
+#ifdef GAMES
+
+        if (w->type == WINDOW_TYPE_GAME) {
+            delwin(w->window_bar);
+            delwin(w->window);
+            delwin(w->game->window);
+            w->window = newwin(LINES, COLS, 0, 0);
+
+            getmaxyx(w->window, y2, x2);
+
+            if (y2 <= 0 || x2 <= 0) {
+                fprintf(stderr, "Failed to resize game window: max_x: %d, max_y: %d\n", x2, y2);
+                delwin(w->window);
+                continue;
+            }
+
+            w->window_bar = subwin(w->window, WINDOW_BAR_HEIGHT, COLS, LINES - 2, 0);
+            w->game->window = subwin(w->window, y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT, x2, 0, 0);
+            continue;
+        }
+
+#endif // GAMES
+
         if (w->help->active) {
             wclear(w->help->win);
         }
@@ -469,14 +555,12 @@ void on_window_resize(void)
 
         w->window = newwin(LINES, COLS, 0, 0);
 
-        int x2;
-        int y2;
         getmaxyx(w->window, y2, x2);
 
         if (y2 <= 0 || x2 <= 0) {
             fprintf(stderr, "Failed to resize window: max_x: %d, max_y: %d\n", x2, y2);
             delwin(w->window);
-            return;
+            continue;
         }
 
         if (w->show_peerlist) {
@@ -706,6 +790,26 @@ void draw_active_window(Tox *m)
     a->onDraw(a, m);
     wrefresh(a->window);
 
+#ifdef GAMES
+
+    if (a->type == WINDOW_TYPE_GAME) {
+        int ch = getch();
+
+        if (ch == ERR) {
+            return;
+        }
+
+        if (ch == user_settings->key_next_tab || ch == user_settings->key_prev_tab) {
+            set_next_window(ch);
+        }
+
+        a->onKey(a, m, ch, false);
+
+        return;
+    }
+
+#endif // GAMES
+
     wint_t ch = 0;
     int printable = get_current_char(&ch);
 
@@ -792,14 +896,35 @@ int get_num_active_windows(void)
 void kill_all_windows(Tox *m)
 {
     for (uint8_t i = 2; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] == NULL) {
+        ToxWindow *w = windows[i];
+
+        if (w == NULL) {
             continue;
         }
 
-        if (windows[i]->type == WINDOW_TYPE_CHAT) {
-            kill_chat_window(windows[i], m);
-        } else if (windows[i]->type == WINDOW_TYPE_CONFERENCE) {
-            free_conference(windows[i], windows[i]->num);
+        switch (w->type) {
+            case WINDOW_TYPE_CHAT: {
+                kill_chat_window(w, m);
+                break;
+            }
+
+            case WINDOW_TYPE_CONFERENCE: {
+                free_conference(w, w->num);
+                break;
+            }
+
+#ifdef GAMES
+
+            case WINDOW_TYPE_GAME: {
+                game_kill(w);
+                break;
+            }
+
+#endif // GAMES
+
+            default: {
+                break;
+            }
         }
     }
 
