@@ -43,6 +43,13 @@
 #define TOP_BAR_HEIGHT 1
 #define WINDOW_BAR_HEIGHT 1
 
+
+typedef enum CustomPacket {
+    CUSTOM_PACKET_GAME_INVITE = 160,
+    CUSTOM_PACKET_GAME_DATA   = 161,
+} CustomPacket;
+
+
 /* ncurses colour pairs as FOREGROUND_BACKGROUND. No background defaults to black. */
 typedef enum {
     WHITE,
@@ -53,8 +60,10 @@ typedef enum {
     YELLOW,
     MAGENTA,
     BLACK,
-    BLUE_BLACK,
     BLACK_WHITE,
+    WHITE_BLACK,
+    WHITE_BLUE,
+    WHITE_GREEN,
     BAR_TEXT,
     STATUS_ONLINE,
     BAR_ACCENT,
@@ -78,6 +87,10 @@ typedef enum {
     WINDOW_TYPE_CHAT,
     WINDOW_TYPE_CONFERENCE,
     WINDOW_TYPE_FRIEND_LIST,
+
+#ifdef GAMES
+    WINDOW_TYPE_GAME,
+#endif
 } WINDOW_TYPE;
 
 /* Fixes text color problem on some terminals.
@@ -130,6 +143,10 @@ typedef struct PromptBuf PromptBuf;
 typedef struct ChatContext ChatContext;
 typedef struct Help Help;
 
+#ifdef GAMES
+typedef struct GameData GameData;
+#endif
+
 struct ToxWindow {
     /* ncurses */
     bool(*onKey)(ToxWindow *, Tox *, wint_t, bool);
@@ -156,6 +173,11 @@ struct ToxWindow {
     void(*onTypingChange)(ToxWindow *, Tox *, uint32_t, bool);
     void(*onReadReceipt)(ToxWindow *, Tox *, uint32_t, uint32_t);
 
+#ifdef GAMES
+    void(*onGameInvite)(ToxWindow *, Tox *, uint32_t, const uint8_t *, size_t);
+    void(*onGameData)(ToxWindow *, Tox *, uint32_t, const uint8_t *, size_t);
+#endif // GAMES
+
 #ifdef AUDIO
 
     void(*onInvite)(ToxWindow *, ToxAV *, uint32_t, int);
@@ -193,6 +215,10 @@ struct ToxWindow {
     StatusBar *stb;
     Help *help;
 
+#ifdef GAMES
+    GameData *game;
+#endif
+
     WINDOW *window;
     WINDOW *window_bar;
 };
@@ -284,4 +310,5 @@ void draw_window_bar(ToxWindow *self);
    call at least once per second */
 void refresh_inactive_windows(void);
 
-#endif /* WINDOWS_H */
+#endif // WINDOWS_H
+
